<?php
namespace nur\sery\schema\_assoc;

use nulib\cl;
use nur\sery\schema\input\Input;
use nur\sery\schema\Schema;
use nur\sery\schema\Value;

class AssocSchema extends Schema {
  /** @var array meta-schema d'un schéma de nature tableau associatif */
  const METASCHEMA = [];

  /**
   * indiquer si $definition est une définition de schéma de nature tableau
   * associatif que {@link normalize()} pourrait normaliser
   */
  static function isa_definition($definition): bool {
    if (!is_array($definition)) return false;
    # nature explicitement spécifiée
    if (array_key_exists("", $definition)) {
      $nature = $definition[""];
      if ($nature === "assoc") return true;
      if (is_array($nature)
        && array_key_exists(0, $nature)
        && $nature[0] === "assoc") {
        return true;
      }
      return false;
    }
    # un tableau associatif
    return !cl::have_num_keys($definition);
  }

  static function normalize($definition, $definitionKey=null): array {
  }

  function __construct($definition=null, $definitionKey=null, bool $normalize=true) {
    if ($definition === null) $definition = static::SCHEMA;
    if ($normalize) $definition = self::normalize($definition, $definitionKey);
    $this->definition = $definition;
  }

  function isAssoc(?AssocSchema &$assoc=null): bool {
    $assoc = $this;
    return true;
  }

  function newValue(?Value &$destv=null, &$dest=null, $destKey=null): Value {
    if (!($destv instanceof AssocValue)) $destv = new AssocValue($this);
    if ($dest instanceof Input) $input = $dest;
    else $input = new Input($dest);
    return $destv->reset($input, $destKey);
  }
}
