<?php
namespace nur\sery\schema\types;

use nur\sery\schema\_scalar\ScalarResult;
use nur\sery\schema\_scalar\ScalarSchema;
use nur\sery\schema\input\Input;
use nur\sery\schema\Result;
use nur\sery\schema\Schema;

class tfloat implements IType {
  static function ensure_float(&$float): void {
    if (!is_float($float)) $float = floatval($float);
  }

  static function ensure_nfloat(&$float): void {
    if ($float !== null) self::ensure_float($float);
  }

  function isAvailable(Input $input, $destKey): bool {
    return $input->isAvailable($destKey) && $input->get($destKey) !== false;
  }

  function isNull($value): bool {
    return $value === null || (is_string($value) && trim($value) === "");
  }

  function isValid($value, ?bool &$normalized=null): bool {
    if (is_string($value)) $valid = is_numeric(trim($value));
    else $valid = is_scalar($value);
    $normalized = is_float($value);
    return $valid;
  }

  /**
   * @var ScalarResult $result
   * @var ScalarSchema $schema
   */
  function verifix(&$value, Result &$result, Schema $schema): bool {
    if (is_float($value)) {
      $result->setNormalized();
      return false;
    } elseif (is_string($value)) {
      $float = trim($value);
      if (is_numeric($float)) $value = floatval($float);
      else return $result->setInvalid($value, $schema);
    } elseif (is_scalar($value)) {
      $value = floatval($value);
    } else {
      return $result->setInvalid($value, $schema);
    }
    $result->setValid();
    return true;
  }

  function format($value, $format=null): string {
    if ($format !== null) return sprintf($format, $value);
    else return strval($value);
  }
}
